import { Component, OnInit } from '@angular/core';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import { ExcelService } from 'src/app/service/excel.service';
import { ConfirmModalComponent } from '../confirm-modal/confirm-modal.component';
import { AlertModalComponent } from '../alert-modal/alert-modal.component';
import { CourseModel } from 'src/app/model/course.model';
import { CourseService } from 'src/app/service/course.service';
import { UploadService } from 'src/app/service/upload.service';
import { TagService } from 'src/app/service/tag.service';
import { TagModel } from 'src/app/model/tag.mmodel';
import { GroupModel } from 'src/app/model/group.mmodel';
import { GroupService } from 'src/app/service/group.service';
import { OpenImageComponent } from '../open-image/open-image.component';
declare var require: any
const FileSaver = require('file-saver');
@Component({
  selector: 'app-list-course',
  templateUrl: './list-course.component.html',
  styleUrls: ['./list-course.component.scss']
})
export class ListCourseComponent implements OnInit {
  page = 1;
  pageSize = 10;
  listCourse:CourseModel[]=[]
  modelCourse:CourseModel = new CourseModel({});
  checkEdit:boolean = false
  search:string = ''
  listTag:TagModel[]=[]
  listGroup:GroupModel[]=[]
  constructor(private modalService: NgbModal,private courseService:CourseService,private uploadService:UploadService,private tagService:TagService,private groupService:GroupService) {
  }
  async onUploadImage(event:any){
    try{
      if (event.target.files.length > 0) {
        let fileData = event.target.files[0];
        const formData = new FormData();
        formData.append('file', fileData);
        const allowedTypes = ['image/png', 'image/jpeg', 'image/jpg'];
        if (!allowedTypes.includes(fileData.type)) {
          this.openAlertModal('อัพโหลดได้เฉพาะไฟล์ *.jpeg, *.jpg, *.png เท่านั้น')
        } else {
          const data = await this.uploadService.uploadImage(formData).toPromise();
          if(data){
            this.modelCourse.thumbnail = data.body.fileId
          }
        }
      }
    } catch (error) {
      console.error('Error loading data:', error);
    }
  }
  onSelectFile(event:any) {
    if (event.target.files && event.target.files[0]) {
      const reader = new FileReader();
      const file: File = event.target.files[0];
      reader.readAsDataURL(event.target.files[0]); // read file as data url
      reader.onload = (event) => { // called once readAsDataURL is completed
        if(event){
          const allowedTypes = ['application/json'];
          if (!allowedTypes.includes(file.type)) {
            this.openAlertModal('อัพโหลดได้เฉพาะไฟล์ *.xlsx เท่านั้น')
          } else {
            let base64 = event.target!.result as string
            this.modelCourse.courseObj = base64.split(',')[1];
          }
       }
      }
    }

  }
 async downloadFile(logId:string){
  try {
    const data = await this.courseService.downloadFile(logId).toPromise();
    if(data){
      FileSaver.saveAs(new Blob([data]), "file_download.json");
    }
  } catch (error) {
    console.error('Error loading data:', error);
  }
  }
  deleteFile(item:CourseModel){
    const modalRef = this.modalService.open(ConfirmModalComponent, {
      centered: true,
      backdrop: 'static',
    })
    modalRef.componentInstance.message = 'คุณต้องการลบข้อมูลหรือไม่'
    modalRef.result.then(result => {
      this.courseService.deleteCourse(item).subscribe(result => {
        if (result) {
          this.openAlertModal('ลบข้อมูลสำเร็จ')
          this.getListCourse();
        } else {
      this.openAlertModal('ไม่สามารถลบข้อมูลได้')
        }
      }, error => {
        this.openAlertModal(error.message)
      })
    }, reject => { })

  }
  openAlertModal(message?: string) {
    const modalRef = this.modalService.open(AlertModalComponent, {
      centered: true,
      backdrop: 'static'
    })
    modalRef.componentInstance.message = message ? message : ""
    modalRef.result.then(result => {
      // this.modalService.dismissAll()
    }, reason => {
      // this.modalService.dismissAll()
    })
  }

  onUpdate(){
    const modalRef = this.modalService.open(ConfirmModalComponent, {
      centered: true,
      backdrop: 'static',
    })
    modalRef.componentInstance.message = 'คุณต้องการอัพเดทข้อมูลหรือไม่'
    modalRef.result.then(result => {
      console.log(this.modelCourse);
      this.courseService.createCourse(this.modelCourse).subscribe(result => {
        if (result) {
          this.modalService.dismissAll()
          this.openAlertModal('อัพเดทข้อมูลสำเร็จ')
          this.getListCourse();
        } else {
      this.openAlertModal('ไม่สามารถอัพเดทข้อมูลได้')
        }
      }, error => {
        this.openAlertModal(error.message)
      })
    }, reject => { })
  }
  onCreate(){
    const modalRef = this.modalService.open(ConfirmModalComponent, {
      centered: true,
      backdrop: 'static',
    })
    modalRef.componentInstance.message = 'คุณต้องการบันทึกข้อมูลหรือไม่'
    modalRef.result.then(result => {
      this.courseService.createCourse(this.modelCourse).subscribe(result => {
        if (result) {
          this.modalService.dismissAll()
          this.openAlertModal('บันทึกข้อมูลสำเร็จ')
          this.getListCourse();
        } else {
      this.openAlertModal('ไม่สามารถสร้างเอกสารได้')
        }
      }, error => {
        this.openAlertModal(error.message)
      })
    }, reject => { })
  }
  onSumit() {
   if(this.checkEdit){
    this.onUpdate();
   }else{
    this.onCreate();
   }
  }
  async getExcelById(targetModal: NgbModal,id:string){
    try {
      const data = await this.courseService.getCourseById(id).toPromise();
      this.modelCourse = new CourseModel(data)
      if(data){
        this.modalService.open(targetModal, {
          centered: true,
          backdrop: 'static',
          size: 'lg'
        });
      }
    } catch (error) {
      console.error('Error loading data:', error);
    }
  }
  openModal(targetModal: NgbModal,item?:CourseModel) {
    if(item){
      this.getExcelById(targetModal,item.logId)
      this.checkEdit = true
    }else{
      this.modelCourse = new CourseModel({});
      this.checkEdit = false
      this.modalService.open(targetModal, {
        centered: true,
        backdrop: 'static',
        size: 'lg'
      });
    }
  }
  async getListCourse(){
    try {
      const data = await this.courseService.getListCourse().toPromise();
      this.listCourse = data.map(x => new CourseModel(x))
    } catch (error) {
      console.error('Error loading data:', error);
    }
  }
  filterListCourse(){
    return this.listCourse.filter(x => x.thName.toLowerCase().includes(this.search.toLowerCase())||x.engName.toLowerCase().includes(this.search.toLowerCase()))
  }
  
  ngOnInit() {
    this.getListCourse();
    this.getListTag();
    this.getListGroup();
  }
  async getListGroup(){
    try{
      const data = await this.groupService.getList().toPromise();
      this.listGroup = data.map(x => new GroupModel(x))
    }catch(error){
      console.error('Error loading data:', error);
    }
  }
  selectGroup(){
    let findData = this.listGroup.find(x => x.groupId == this.modelCourse.group.groupId) 
    this.modelCourse.group = findData?new GroupModel(findData):new GroupModel({})
  }

  closeBtnClick() {
    this.modalService.dismissAll();
  }
  async getListTag(){
    try{
      const data = await this.tagService.getList().toPromise();
      this.listTag = data.map(x => new TagModel(x))
    }catch(error){
      console.error('Error loading data:', error);
    }
  }
  openEmployeeModal(image:string) {
    const modalRef = this.modalService.open(OpenImageComponent, {
      centered: true,
      windowClass: 'my-dialog-img-preview'
    })
    modalRef.componentInstance.linkImage = image
    modalRef.result.then(result => {
    }, reason => {
      this.modalService.dismissAll()
    })
  }
  openLink(url:string){
    window.open(url, "_blank");
  }
}
