import { Component, OnInit } from '@angular/core';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import { ConfirmModalComponent } from '../confirm-modal/confirm-modal.component';
import { AlertModalComponent } from '../alert-modal/alert-modal.component';
import { DocumentService } from 'src/app/service/document.service';
import { DocumentModel } from 'src/app/model/document.model';
import { UploadService } from 'src/app/service/upload.service';
import { TagService } from 'src/app/service/tag.service';
import { TagModel } from 'src/app/model/tag.mmodel';
import { GroupModel } from 'src/app/model/group.mmodel';
import { GroupService } from 'src/app/service/group.service';
import { OpenImageComponent } from '../open-image/open-image.component';
declare var require: any
const FileSaver = require('file-saver');
@Component({
  selector: 'app-list-doc',
  templateUrl: './list-doc.component.html',
  styleUrls: ['./list-doc.component.scss']
})
export class ListDocComponent implements OnInit {
  page = 1;
  pageSize = 10;
  listDoc: DocumentModel[] = []
  modelDoc: DocumentModel = new DocumentModel({});
  checkEdit: boolean = false
  search: string = ''
  listTag: TagModel[] = []
  listGroup: GroupModel[] = []
  constructor(private modalService: NgbModal, private documentService: DocumentService, private uploadService: UploadService, private tagService: TagService, private groupService: GroupService) {
  }
  async onUploadImage(event: any) {
    try {
      if (event.target.files.length > 0) {
        let fileData = event.target.files[0];
        const formData = new FormData();
        formData.append('file', fileData);
        const allowedTypes = ['image/png', 'image/jpeg', 'image/jpg'];
        if (!allowedTypes.includes(fileData.type)) {
          this.openAlertModal('อัพโหลดได้เฉพาะไฟล์ *.jpeg, *.jpg, *.png เท่านั้น')
        } else {
          const data = await this.uploadService.uploadImage(formData).toPromise();
          if (data) {
            this.modelDoc.thumbnail = data.body.fileId
          }
        }
      }
    } catch (error) {
      console.error('Error loading data:', error);
    }
  }
  onSelectFile(event: any, lang: string) {
    if (event.target.files && event.target.files[0]) {
      const reader = new FileReader();
      const file: File = event.target.files[0];
      reader.readAsDataURL(event.target.files[0]); // read file as data url
      reader.onload = (event) => { // called once readAsDataURL is completed
        if (event) {
          const allowedTypes = ['application/vnd.openxmlformats-officedocument.wordprocessingml.document', 'application/msword'];
          if (!allowedTypes.includes(file.type)) {
            this.openAlertModal('อัพโหลดได้เฉพาะไฟล์ *.doc เท่านั้น')
          } else {
            let base64 = event.target!.result as string
            if (lang == 'th') {
              this.modelDoc.thDocObj = base64.split(',')[1];
            } else {
              this.modelDoc.engDocObj = base64.split(',')[1];
            }
          }
        }
      }
    }
  }
  async downloadFile(logId: string, lang: string) {
    try {
      const data = await this.documentService.downloadFile(logId, lang).toPromise();
      if (data) {
        FileSaver.saveAs(new Blob([data]), "file_download.doc");
      }
    } catch (error) {
      console.error('Error loading data:', error);
    }
  }
  deleteFile(item: DocumentModel) {
    const modalRef = this.modalService.open(ConfirmModalComponent, {
      centered: true,
      backdrop: 'static',
    })
    modalRef.componentInstance.message = 'คุณต้องการลบข้อมูลหรือไม่'
    modalRef.result.then(result => {
      this.documentService.deleteExcel(item).subscribe(result => {
        if (result) {
          this.openAlertModal('ลบข้อมูลสำเร็จ')
          this.getListDoc();
        } else {
          this.openAlertModal('ไม่สามารถลบข้อมูลได้')
        }
      }, error => {
        this.openAlertModal(error.message)
      })
    }, reject => { })
  }
  openAlertModal(message?: string) {
    const modalRef = this.modalService.open(AlertModalComponent, {
      centered: true,
      backdrop: 'static'
    })
    modalRef.componentInstance.message = message ? message : ""
    modalRef.result.then(result => {
      // this.modalService.dismissAll()
    }, reason => {
      // this.modalService.dismissAll()
    })
  }

  onUpdate() {
    const modalRef = this.modalService.open(ConfirmModalComponent, {
      centered: true,
      backdrop: 'static',
    })
    modalRef.componentInstance.message = 'คุณต้องการอัพเดทข้อมูลหรือไม่'
    modalRef.result.then(result => {
      this.documentService.createDoc(this.modelDoc).subscribe(result => {
        this.modalService.dismissAll()
        this.openAlertModal('อัพเดทข้อมูลสำเร็จ')
        this.getListDoc();
        // if (result) {
        //   this.modalService.dismissAll()
        //   this.openAlertModal('อัพเดทข้อมูลสำเร็จ')
        //   this.getListDoc();
        // } else {
        //   this.openAlertModal('ไม่สามารถอัพเดทข้อมูลได้')
        // }
      }, error => {
        this.openAlertModal(error.message)
      })
    }, reject => { })
  }
  onCreate() {
    const modalRef = this.modalService.open(ConfirmModalComponent, {
      centered: true,
      backdrop: 'static',
    })
    modalRef.componentInstance.message = 'คุณต้องการบันทึกข้อมูลหรือไม่'
    modalRef.result.then(result => {
      this.documentService.createDoc(this.modelDoc).subscribe(result => {
        if (result) {
          this.modalService.dismissAll()
          this.openAlertModal('บันทึกข้อมูลสำเร็จ')
          this.getListDoc();
        } else {
          this.openAlertModal('ไม่สามารถสร้างเอกสารได้')
        }
      }, error => {
        this.openAlertModal(error.message)
      })
    }, reject => { })
  }
  onSumit() {
    if (this.checkEdit) {
      this.onUpdate();
    } else {
      this.onCreate();
    }
  }
  async getExcelById(targetModal: NgbModal, id: string) {
    try {
      const data = await this.documentService.getDocById(id).toPromise();
      this.modelDoc = new DocumentModel(data)
      if (data) {
        this.modalService.open(targetModal, {
          centered: true,
          backdrop: 'static',
          size: 'lg'
        });
      }
    } catch (error) {
      console.error('Error loading data:', error);
    }
  }
  openModal(targetModal: NgbModal, item?: DocumentModel) {
    if (item) {
      this.getExcelById(targetModal, item.logId)
      this.checkEdit = true
    } else {
      this.modelDoc = new DocumentModel({});
      this.checkEdit = false
      this.modalService.open(targetModal, {
        centered: true,
        backdrop: 'static',
        size: 'lg'
      });
    }

  }
  getStatus(status: string) {
    if (status == '0') {
      return 'รออนุมัติ'
    } else if (status == '1') {
      return 'เปิดใช้งาน'
    } else if (status == '2') {
      return 'ไม่อนุมัติ'
    }
  }
  async getListDoc() {
    try {
      const data = await this.documentService.getListDoc().toPromise();
      this.listDoc = data.map(x => new DocumentModel(x))
    } catch (error) {
      console.error('Error loading data:', error);
    }
  }
  filterListDoc() {
    return this.listDoc.filter(x => x.thName.toLowerCase().includes(this.search.toLowerCase()) || x.engName.toLowerCase().includes(this.search.toLowerCase()))
  }

  ngOnInit() {
    this.getListDoc();
    this.getListTag();
    this.getListGroup();
  }
  async getListGroup() {
    try {
      const data = await this.groupService.getList().toPromise();
      this.listGroup = data.map(x => new GroupModel(x))
    } catch (error) {
      console.error('Error loading data:', error);
    }
  }
  selectGroup() {
    let findData = this.listGroup.find(x => x.groupId == this.modelDoc.group.groupId)
    this.modelDoc.group = findData ? new GroupModel(findData) : new GroupModel({})
  }
  async getListTag() {
    try {
      const data = await this.tagService.getList().toPromise();
      this.listTag = data.map(x => new TagModel(x))
    } catch (error) {
      console.error('Error loading data:', error);
    }
  }
  openEmployeeModal(image:string) {
    const modalRef = this.modalService.open(OpenImageComponent, {
      centered: true,
      windowClass: 'my-dialog-img-preview'
    })
    modalRef.componentInstance.linkImage = image
    modalRef.result.then(result => {
    }, reason => {
      this.modalService.dismissAll()
    })
  }
  closeBtnClick() {
    this.modalService.dismissAll();
  }
  openLink(url: string) {
    window.open(url, "_blank");
  }
}
