import { Component, OnInit } from '@angular/core';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import { ExcelModel } from 'src/app/model/excel.model';
import { ExcelService } from 'src/app/service/excel.service';
import { ConfirmModalComponent } from '../confirm-modal/confirm-modal.component';
import { AlertModalComponent } from '../alert-modal/alert-modal.component';
import { UploadService } from 'src/app/service/upload.service';
import { TagService } from 'src/app/service/tag.service';
import { TagModel } from 'src/app/model/tag.mmodel';
import { GroupService } from 'src/app/service/group.service';
import { GroupModel } from 'src/app/model/group.mmodel';
import { OpenImageComponent } from '../open-image/open-image.component';
declare var require: any
const FileSaver = require('file-saver');

@Component({
  selector: 'app-list-excel',
  templateUrl: './list-excell.component.html',
  styleUrls: ['./list-excell.component.scss']
})
export class ListExcelComponent implements OnInit {
  page = 1;
  pageSize = 10;
  listExcel:ExcelModel[]=[]
  modelExcel:ExcelModel = new ExcelModel({});
  checkEdit:boolean = false
  search:string = ''
  
  listTag:TagModel[]=[]
  listGroup:GroupModel[]=[]
  constructor(private modalService: NgbModal,private excelService:ExcelService,private uploadService:UploadService ,private tagService:TagService,private groupService:GroupService) {
  }
 async onUploadImage(event:any){
  try{
    if (event.target.files.length > 0) {
      let fileData = event.target.files[0];
      const formData = new FormData();
      formData.append('file', fileData);
      const allowedTypes = ['image/png', 'image/jpeg', 'image/jpg'];
      if (!allowedTypes.includes(fileData.type)) {
        this.openAlertModal('อัพโหลดได้เฉพาะไฟล์ *.jpeg, *.jpg, *.png เท่านั้น')
      } else {
        const data = await this.uploadService.uploadImage(formData).toPromise();
        if(data){
          this.modelExcel.thumbnail = data.body.fileId
        }
      }
    }
  } catch (error) {
    console.error('Error loading data:', error);
  }
}
  async getListTag(){
    try{
      const data = await this.tagService.getList().toPromise();
      this.listTag = data.map(x => new TagModel(x))
    }catch(error){
      console.error('Error loading data:', error);
    }
  }
  async getListGroup(){
    try{
      const data = await this.groupService.getList().toPromise();
      this.listGroup = data.map(x => new GroupModel(x))
    }catch(error){
      console.error('Error loading data:', error);
    }
  }
  selectGroup(){
    let findData = this.listGroup.find(x => x.groupId == this.modelExcel.group.groupId) 
    this.modelExcel.group = findData?new GroupModel(findData):new GroupModel({})
  }
  onSelectFile(event:any) {
    if (event.target.files && event.target.files[0]) {
      const reader = new FileReader();
      const file: File = event.target.files[0];
      console.log("🚀 ~ ListExcelComponent ~ onSelectFile ~ file:", file)
      reader.readAsDataURL(event.target.files[0]); // read file as data url
      reader.onload = (event) => { // called once readAsDataURL is completed
        if(event){
          const allowedTypes = ['application/vnd.openxmlformats-officedocument.spreadsheetml.sheet','application/vnd.ms-excel.sheet.macroEnabled.12'];
          if (!allowedTypes.includes(file.type)) {
            this.openAlertModal('อัพโหลดได้เฉพาะไฟล์ *.xlsx *.xlsm เท่านั้น')
          } else {
            let base64 = event.target!.result as string
            this.modelExcel.excelObj = base64.split(',')[1];
            this.modelExcel.fileType = file.name.split('.')[file.name.split('.').length-1]
          }
       }
      }
    }
  }
  async downloadFile(logId:string){
    try {
      const data = await this.excelService.downloadFile(logId).toPromise();
      if(data){
        FileSaver.saveAs(new Blob([data]), "file_download.xlsx");
      }
    } catch (error) {
      console.error('Error loading data:', error);
    }
  }


  deleteFile(item:ExcelModel){
    const modalRef = this.modalService.open(ConfirmModalComponent, {
      centered: true,
      backdrop: 'static',
    })
    modalRef.componentInstance.message = 'คุณต้องการลบข้อมูลหรือไม่'
    modalRef.result.then(result => {
      this.excelService.deleteExcel(item).subscribe(result => {
        if (result) {
          this.openAlertModal('ลบข้อมูลสำเร็จ')
          this.getListExcel();
        } else {
      this.openAlertModal('ไม่สามารถลบข้อมูลได้')
        }
      }, error => {
        this.openAlertModal(error.message)
      })
    }, reject => { })
  }
  openAlertModal(message?: string) {
    const modalRef = this.modalService.open(AlertModalComponent, {
      centered: true,
      backdrop: 'static'
    })
    modalRef.componentInstance.message = message ? message : ""
    modalRef.result.then(result => {
      // this.modalService.dismissAll()
    }, reason => {
      // this.modalService.dismissAll()
    })
  }

  onUpdate(){
    const modalRef = this.modalService.open(ConfirmModalComponent, {
      centered: true,
      backdrop: 'static',
    })
    modalRef.componentInstance.message = 'คุณต้องการอัพเดทข้อมูลหรือไม่'
    modalRef.result.then(result => {
      console.log(this.modelExcel);
      this.excelService.createExcel(this.modelExcel).subscribe(result => {
        if (result) {
          this.modalService.dismissAll()
          this.openAlertModal('อัพเดทข้อมูลสำเร็จ')
          this.getListExcel();
        } else {
      this.openAlertModal('ไม่สามารถอัพเดทข้อมูลได้')
        }
      }, error => {
        this.openAlertModal(error.message)
      })
    }, reject => { })
  }
  onCreate(){
    const modalRef = this.modalService.open(ConfirmModalComponent, {
      centered: true,
      backdrop: 'static',
    })
    modalRef.componentInstance.message = 'คุณต้องการบันทึกข้อมูลหรือไม่'
    modalRef.result.then(result => {
      this.excelService.createExcel(this.modelExcel).subscribe(result => {
        if (result) {
          this.modalService.dismissAll()
          this.openAlertModal('บันทึกข้อมูลสำเร็จ')
          this.getListExcel();
        } else {
      this.openAlertModal('ไม่สามารถสร้างเอกสารได้')
        }
      }, error => {
        this.openAlertModal(error.message)
      })
    }, reject => { })
  }
  onSumit() {
   if(this.checkEdit){
    this.onUpdate();
   }else{
    this.onCreate();
   }
  }
  async getExcelById(targetModal: NgbModal,id:string){
    try {
      const data = await this.excelService.getExcelById(id).toPromise();
      this.modelExcel = new ExcelModel(data)
      console.log("🚀 ~ ListExcelComponent ~ getExcelById ~ this.modelExcel:", this.modelExcel)
      if(data){
        this.modalService.open(targetModal, {
          centered: true,
          backdrop: 'static',
          size: 'lg'
        });
      }
    } catch (error) {
      console.error('Error loading data:', error);
    }
  }
  openModal(targetModal: NgbModal,item?:ExcelModel) {
    if(item){
      this.getExcelById(targetModal,item.logId)
      this.checkEdit = true
    }else{
      this.modelExcel = new ExcelModel({});
      this.checkEdit = false
      this.modalService.open(targetModal, {
        centered: true,
        backdrop: 'static',
        size: 'lg'
      });
    }
   
  }
  openLink(url:string){
    window.open(url, "_blank");
  }
  getStatus(status:string){
    if(status == '0'){
      return 'รออนุมัติ'
    } else if(status == '1'){
      return 'เปิดใช้งาน'
    } else if(status == '2'){
      return 'ไม่อนุมัติ'
    }
  }
  async getListExcel(){
    try {
      const data = await this.excelService.getListExcel().toPromise();
      this.listExcel = data.map(x => new ExcelModel(x))
    } catch (error) {
      console.error('Error loading data:', error);
    }
  }
  filterListExcel(){
    return this.listExcel.filter(x => x.thName.toLowerCase().includes(this.search.toLowerCase())||x.engName.toLowerCase().includes(this.search.toLowerCase()))
  }
  
  ngOnInit() {
    this.getListExcel();
    this.getListTag();
    this.getListGroup();
  }

  openEmployeeModal(image:string) {
    const modalRef = this.modalService.open(OpenImageComponent, {
      centered: true,
      windowClass: 'my-dialog-img-preview'
    })
    modalRef.componentInstance.linkImage = image
    modalRef.result.then(result => {
    }, reason => {
      this.modalService.dismissAll()
    })
  }
  closeBtnClick() {
    this.modalService.dismissAll();
  }
  coverDate(date:string){
    return  date.split('-').reverse().join('/')
    }
}
